import { parseExpression } from '@babel/parser';
import { language } from './highlight';
import { loadPrettier } from './format';
export var registerExpression = function (languageId, editor, monaco) {
    if (languageId !== 'javascript.expression' &&
        languageId !== 'typescript.expression')
        return;
    monaco.languages.register({ id: languageId });
    monaco.languages.setMonarchTokensProvider(languageId, language);
    loadPrettier().then(function (_a) {
        var format = _a.format;
        monaco.languages.registerDocumentFormattingEditProvider(languageId, {
            provideDocumentFormattingEdits: function (model) {
                return [
                    {
                        text: format === null || format === void 0 ? void 0 : format(model.getValue(), {
                            semi: false,
                            parser: function (text) {
                                return parseExpression(text);
                            },
                        }),
                        range: model.getFullModelRange(),
                    },
                ];
            },
        });
    });
    var validate = function () {
        var currentLanguage = editor.getModel()['_languageIdentifier'].language;
        if (currentLanguage !== languageId)
            return;
        try {
            parseExpression(editor.getValue(), {
                plugins: ['typescript'],
            });
            monaco.editor.setModelMarkers(editor.getModel(), languageId, []);
        }
        catch (e) {
            monaco.editor.setModelMarkers(editor.getModel(), languageId, [
                {
                    code: '1003',
                    severity: 8,
                    startLineNumber: e.loc.line,
                    startColumn: e.loc.column,
                    endLineNumber: e.loc.line,
                    endColumn: e.loc.column,
                    message: e.message,
                },
            ]);
        }
    };
    var requestId = null;
    editor.onDidChangeModelContent(function () {
        clearTimeout(requestId);
        requestId = setTimeout(validate, 1000);
    });
};
