var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import React, { useState, useEffect, useRef } from 'react';
import Editor from '@monaco-editor/react';
import { usePrefix, useTheme } from '@designable/react';
import { registerExpression } from './expression';
import { format } from './format';
import chromeTheme from './themes/chrome';
import monokaiTheme from './themes/monokai';
import cls from 'classnames';
import './styles.less';
export var MonacoInput = function (_a) {
    var className = _a.className, language = _a.language, defaultLanguage = _a.defaultLanguage, width = _a.width, height = _a.height, onMount = _a.onMount, onChange = _a.onChange, props = __rest(_a, ["className", "language", "defaultLanguage", "width", "height", "onMount", "onChange"]);
    var _b = __read(useState(''), 2), errors = _b[0], setErrors = _b[1];
    var _c = __read(useState(false), 2), mounted = _c[0], setMounted = _c[1];
    var input = props.value || props.defaultValue;
    var _d = __read(useState(input), 2), value = _d[0], setValue = _d[1];
    var theme = useTheme();
    var valueRef = useRef('');
    var monacoRef = useRef();
    var changedRef = useRef(false);
    var editorRef = useRef();
    var prefix = usePrefix('monaco-input');
    useEffect(function () {
        if (input !== undefined) {
            format(input, language || defaultLanguage).then(function (prettyCode) {
                setValue(prettyCode);
            }, function (e) {
                setErrors(e === null || e === void 0 ? void 0 : e.message);
            });
        }
    }, [input]);
    useEffect(function () {
        if (monacoRef.current) {
            monacoRef.current.editor.setTheme(theme === 'dark' ? 'monokai' : 'chrome-devtools');
        }
    }, [theme, mounted]);
    valueRef.current = value;
    return (React.createElement("div", { className: cls(prefix, className), style: { width: width, height: height } },
        React.createElement(Editor, __assign({}, props, { defaultLanguage: defaultLanguage, language: language, value: value, width: "100%", height: "100%", onMount: function (editor, monaco) {
                monacoRef.current = monaco;
                editorRef.current = editor;
                onMount === null || onMount === void 0 ? void 0 : onMount(editor, monaco);
                registerExpression(language, editor, monaco);
                monaco.editor.defineTheme('monokai', monokaiTheme);
                monaco.editor.defineTheme('chrome-devtools', chromeTheme);
                monaco.languages.typescript.typescriptDefaults.setCompilerOptions({
                    target: monaco.languages.typescript.ScriptTarget.Latest,
                    allowNonTsExtensions: true,
                    moduleResolution: monaco.languages.typescript.ModuleResolutionKind.NodeJs,
                    module: monaco.languages.typescript.ModuleKind.CommonJS,
                    noEmit: true,
                    esModuleInterop: true,
                    jsx: monaco.languages.typescript.JsxEmit.React,
                    reactNamespace: 'React',
                    allowJs: true,
                });
                monaco.languages.typescript.typescriptDefaults.setDiagnosticsOptions({
                    noSemanticValidation: false,
                    noSyntaxValidation: false,
                });
                setMounted(true);
            }, onValidate: function (markers) {
                if (markers.length) {
                    var marker = markers[0];
                    setErrors("[" + marker.code + "]: " + marker.message + "  " + marker.startLineNumber + ":" + marker.startColumn);
                }
                else if (changedRef.current) {
                    onChange === null || onChange === void 0 ? void 0 : onChange(valueRef.current);
                    setErrors('');
                }
            }, onChange: function (value) {
                changedRef.current = true;
                valueRef.current = value;
            } })),
        errors && (React.createElement("div", { className: prefix + '-errors' },
            React.createElement("code", null,
                React.createElement("pre", null, errors))))));
};
