"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerExpression = void 0;
var parser_1 = require("@babel/parser");
var highlight_1 = require("./highlight");
var format_1 = require("./format");
var registerExpression = function (languageId, editor, monaco) {
    if (languageId !== 'javascript.expression' &&
        languageId !== 'typescript.expression')
        return;
    monaco.languages.register({ id: languageId });
    monaco.languages.setMonarchTokensProvider(languageId, highlight_1.language);
    format_1.loadPrettier().then(function (_a) {
        var format = _a.format;
        monaco.languages.registerDocumentFormattingEditProvider(languageId, {
            provideDocumentFormattingEdits: function (model) {
                return [
                    {
                        text: format === null || format === void 0 ? void 0 : format(model.getValue(), {
                            semi: false,
                            parser: function (text) {
                                return parser_1.parseExpression(text);
                            },
                        }),
                        range: model.getFullModelRange(),
                    },
                ];
            },
        });
    });
    var validate = function () {
        var currentLanguage = editor.getModel()['_languageIdentifier'].language;
        if (currentLanguage !== languageId)
            return;
        try {
            parser_1.parseExpression(editor.getValue(), {
                plugins: ['typescript'],
            });
            monaco.editor.setModelMarkers(editor.getModel(), languageId, []);
        }
        catch (e) {
            monaco.editor.setModelMarkers(editor.getModel(), languageId, [
                {
                    code: '1003',
                    severity: 8,
                    startLineNumber: e.loc.line,
                    startColumn: e.loc.column,
                    endLineNumber: e.loc.line,
                    endColumn: e.loc.column,
                    message: e.message,
                },
            ]);
        }
    };
    var requestId = null;
    editor.onDidChangeModelContent(function () {
        clearTimeout(requestId);
        requestId = setTimeout(validate, 1000);
    });
};
exports.registerExpression = registerExpression;
